@@ -86,19 +86,13 @@ struct radv_graphics_pipeline {
 
    VkShaderStageFlags active_stages;
 
-   uint32_t spi_shader_col_format;
-   uint32_t cb_shader_mask;
-
    struct radv_dynamic_state dynamic_state;
 
-   struct radv_vs_input_state vs_input_state;
+   struct radv_vertex_input_state vertex_input;
 
    struct radv_multisample_state ms;
    struct radv_ia_multi_vgt_param_helpers ia_multi_vgt_param;
    uint32_t binding_stride[MAX_VBS];
-   uint8_t attrib_bindings[MAX_VERTEX_ATTRIBS];
-   uint32_t attrib_ends[MAX_VERTEX_ATTRIBS];
-   uint32_t attrib_index_offset[MAX_VERTEX_ATTRIBS];
    uint32_t db_render_control;
 
    /* Last pre-PS API stage */
@@ -124,6 +118,9 @@ struct radv_graphics_pipeline {
 
    /* For relocation of shaders with RGP. */
    struct radv_sqtt_shaders_reloc *sqtt_shaders_reloc;
+
+   /* Whether the pipeline imported binaries. */
+   bool has_pipeline_binaries;
 };
 
 RADV_DECL_PIPELINE_DOWNCAST(graphics, RADV_PIPELINE_GRAPHICS)
@@ -563,8 +560,6 @@ radv_normalize_blend_factor(VkBlendOp op, VkBlendFactor *src_factor, VkBlendFact
 void radv_blend_remove_dst(VkBlendOp *func, VkBlendFactor *src_factor, VkBlendFactor *dst_factor,
                            VkBlendFactor expected_dst, VkBlendFactor replacement_src);
 
-unsigned radv_compact_spi_shader_col_format(uint32_t spi_shader_col_format);
-
 unsigned radv_format_meta_fs_key(struct radv_device *device, VkFormat format);
 
 struct radv_ia_multi_vgt_param_helpers radv_compute_ia_multi_vgt_param(const struct radv_device *device,
@@ -598,8 +593,8 @@ struct radv_ps_epilog_key radv_generate_ps_epilog_key(const struct radv_device *
 void radv_graphics_shaders_compile(struct radv_device *device, struct vk_pipeline_cache *cache,
                                    struct radv_shader_stage *stages, const struct radv_graphics_state_key *gfx_state,
                                    bool keep_executable_info, bool keep_statistic_info, bool is_internal,
-                                   struct radv_retained_shaders *retained_shaders, bool noop_fs,
-                                   struct radv_shader **shaders, struct radv_shader_binary **binaries,
+                                   bool skip_shaders_cache, struct radv_retained_shaders *retained_shaders,
+                                   bool noop_fs, struct radv_shader **shaders, struct radv_shader_binary **binaries,
                                    struct radv_shader **gs_copy_shader, struct radv_shader_binary **gs_copy_binary);
 
 struct radv_vgt_shader_key {
@@ -641,4 +636,26 @@ void radv_destroy_graphics_pipeline(struct radv_device *device, struct radv_grap
 
 void radv_destroy_graphics_lib_pipeline(struct radv_device *device, struct radv_graphics_lib_pipeline *pipeline);
 
+struct radv_graphics_pipeline_state {
+   struct vk_graphics_pipeline_state vk;
+   void *vk_data;
+
+   bool compilation_required;
+
+   struct radv_shader_stage *stages;
+
+   struct radv_graphics_pipeline_key key;
+
+   struct radv_pipeline_layout layout;
+};
+
+void radv_graphics_pipeline_hash(const struct radv_device *device, const struct radv_graphics_pipeline_state *gfx_state,
+                                 unsigned char *hash);
+
+VkResult radv_generate_graphics_pipeline_state(struct radv_device *device,
+                                               const VkGraphicsPipelineCreateInfo *pCreateInfo,
+                                               struct radv_graphics_pipeline_state *gfx_state);
+
+void radv_graphics_pipeline_state_finish(struct radv_device *device, struct radv_graphics_pipeline_state *gfx_state);
+
 #endif /* RADV_PIPELINE_GRAPHICS_H */
